package com.ruoyi.web.controller.tool;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Collections;
import java.util.Optional;
import java.util.stream.Collectors;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpMethod;
import org.springframework.http.ResponseEntity;
import org.springframework.http.client.HttpComponentsClientHttpRequestFactory;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.client.RestTemplate;
import org.springframework.web.context.request.async.DeferredResult;
import com.ruoyi.common.annotation.Anonymous;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.R;
import com.ruoyi.common.utils.StringUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import javax.annotation.PostConstruct;

/**
 * 斑马订单控制器
 *
 * @author ruoyi
 */
@Api("斑马订单接口")
@RestController
@RequestMapping("/tool/banma")
@Anonymous
public class BanmaOrderController extends BaseController {
    private static String AUTH_TOKEN = "Bearer e5V8Vlaf9xh5i31xaI300wbdXEE3iLtgip+JXfzZsb7GShP2XCGhoVzTEVxyc8LH";
    private static final String LOGIN_URL = "https://banma365.cn/api/login";
    private static final String LOGIN_USERNAME = "大赢家网络科技(主账号）";
    private static final String LOGIN_PASSWORD = "banma123456";
    private static final String API_URL = "https://banma365.cn/api/order/list?recipientName=&page=%d&size=%d&markFlag=0&state=4&_t=%d";
    private static final String API_URL_WITH_TIME = "https://banma365.cn/api/order/list?recipientName=&page=%d&size=%d&markFlag=0&state=4&orderedAtStart=%s&orderedAtEnd=%s&_t=%d";
    private static final String TRACKING_URL = "https://banma365.cn/zebraExpressHub/web/tracking/getByExpressNumber/%s";
    private static final int CONNECTION_TIMEOUT = 999999999;
    private static final int READ_TIMEOUT = 999999999;
    private static final int DEFAULT_PAGE_SIZE = 20;

    @Autowired
    private RestTemplate restTemplate;

    @Autowired
    private SagawaExpressController sagawaExpressController;

    public BanmaOrderController() {
        HttpComponentsClientHttpRequestFactory factory = new HttpComponentsClientHttpRequestFactory();
        factory.setConnectTimeout(CONNECTION_TIMEOUT);
        factory.setReadTimeout(READ_TIMEOUT);
        restTemplate = new RestTemplate(factory);
    }
    
    /**
     * 初始化方法，启动时刷新token
     */
    @PostConstruct
    public void init() {
        refreshToken();
    }
    @Scheduled(fixedRate = 86400000 * 3)
    public void refreshToken() {
        try {
            // 1. 输入准备：构建请求参数
            Map<String, String> loginParams = new HashMap<>();
            loginParams.put("username", LOGIN_USERNAME);
            loginParams.put("password", LOGIN_PASSWORD);
            
            HttpHeaders headers = new HttpHeaders();
            headers.set("Content-Type", "application/json");
            ResponseEntity<Map> response = restTemplate.postForEntity(
                LOGIN_URL, 
                new HttpEntity<>(loginParams, headers), 
                Map.class
            );
            Optional.ofNullable(response.getBody())
                .filter(body -> Integer.valueOf(0).equals(body.get("code")))
                .map(body -> (Map<String, Object>) body.get("data"))
                .map(data -> (String) data.get("token"))
                .filter(StringUtils::isNotEmpty)
                .ifPresent(token -> {
                    AUTH_TOKEN = "Bearer " + token;
                    logger.info("斑马token刷新成功: {}", token);
                });
        } catch (Exception e) {
            logger.error("斑马token刷新异常: {}", e.getMessage());
        }
    }

    /**
     * 创建HTTP请求实体
     */
    private HttpEntity<String> createHttpEntity() {
        HttpHeaders headers = new HttpHeaders();
        headers.set("Authorization", AUTH_TOKEN);
        return new HttpEntity<>(headers);
    }

    /**
     * 处理订单数据
     */
    @SuppressWarnings("unchecked")
    private Map<String, Object> processOrderData(Map<String, Object> order) {
        if (order == null) return null;

        Map<String, Object> simplifiedOrder = new HashMap<>();
        
        // 提取国际运单号和运费
        String trackingNumber = (String) order.get("internationalTrackingNumber");
        simplifiedOrder.put("internationalTrackingNumber", trackingNumber);
        simplifiedOrder.put("internationalShippingFee", order.get("internationalShippingFee"));

        // 获取物流轨迹信息
        if (StringUtils.isNotEmpty(trackingNumber)) {
            simplifiedOrder.put("trackInfo", getTrackingInfo(trackingNumber));
        }

        // 处理子订单信息
        Optional.ofNullable(order.get("subOrders"))
            .map(subOrders -> (List<Map<String, Object>>) subOrders)
            .filter(list -> !list.isEmpty())
            .map(list -> list.get(0))
            .ifPresent(subOrder -> extractSubOrderFields(simplifiedOrder, subOrder));

        return simplifiedOrder;
    }

    /**
     * 提取子订单字段
     */
    private void extractSubOrderFields(Map<String, Object> simplifiedOrder, Map<String, Object> subOrder) {
        // 基础信息
        simplifiedOrder.put("poTrackingNumber", subOrder.get("poTrackingNumber"));
        simplifiedOrder.put("orderedAt", subOrder.get("orderedAt"));
        simplifiedOrder.put("timeSinceOrder", subOrder.get("timeSinceOrder"));
        simplifiedOrder.put("productImage", subOrder.get("productImage"));
        simplifiedOrder.put("createdAt", subOrder.get("createdAt"));
        
        // 商品信息
        simplifiedOrder.put("productTitle", subOrder.get("productTitle"));
        simplifiedOrder.put("shopOrderNumber", subOrder.get("shopOrderNumber"));
        simplifiedOrder.put("priceJpy", subOrder.get("priceJpy"));
        simplifiedOrder.put("productQuantity", subOrder.get("productQuantity"));
        simplifiedOrder.put("shippingFeeJpy", subOrder.get("shippingFeeJpy"));
        simplifiedOrder.put("productNumber", subOrder.get("productNumber"));
        
        // 采购信息
        simplifiedOrder.put("poNumber", subOrder.get("poNumber"));
        simplifiedOrder.put("shippingFeeCny", subOrder.get("shippingFeeCny"));
        simplifiedOrder.put("poLogisticsCompany", subOrder.get("poLogisticsCompany"));
    }

    /**
     * 获取斑马订单数据
     */
    @SuppressWarnings("unchecked")
    private R<List<Map<String, Object>>> fetchOrdersFromApi(int page, int size, String startDate, String endDate) {
        try {
            HttpEntity<String> entity = createHttpEntity();
            String url = buildApiUrl(page, size, startDate, endDate);
            
            ResponseEntity<Map> response = restTemplate.exchange(url, HttpMethod.GET, entity, Map.class);
            Map<String, Object> responseBody = response.getBody();
            
            if (responseBody == null || !responseBody.containsKey("data")) {
                return R.ok(Collections.emptyList());
            }

            Map<String, Object> dataMap = (Map<String, Object>) responseBody.get("data");
            List<Map<String, Object>> orders = Optional.ofNullable(dataMap.get("list"))
                .map(list -> (List<Map<String, Object>>) list)
                .orElse(Collections.emptyList());
            
            return R.ok(orders.stream()
                .map(this::processOrderData)
                .filter(order -> order != null)
                .collect(Collectors.toList()));
        } catch (Exception e) {
            logger.error("获取订单数据失败: {}", e.getMessage());
            return R.fail("获取订单失败: " + e.getMessage());
        }
    }
    
    /**
     * 构建API URL
     */
    private String buildApiUrl(int page, int size, String startDate, String endDate) {
        if (StringUtils.isNotEmpty(startDate) && StringUtils.isNotEmpty(endDate)) {
            String startTime = startDate + " 00:00:00";
            String endTime = endDate + " 23:59:59";
            return String.format(API_URL_WITH_TIME, page, size, startTime, endTime, System.currentTimeMillis());
        }
        return String.format(API_URL, page, size, System.currentTimeMillis());
    }

    /**
     * 获取物流轨迹信息
     */
    @SuppressWarnings("unchecked")
    private String getTrackingInfo(String trackingNumber) {
        try {
            R<Map<String, Object>> sagawaResult = sagawaExpressController.getTrackingInfo(trackingNumber);
            if (sagawaResult != null && sagawaResult.getCode() == 200) {
                Map<String, Object> sagawaData = sagawaResult.getData();
                if (sagawaData != null && "success".equals(sagawaData.get("status"))) {
                    Map<String, String> trackInfo = (Map<String, String>) sagawaData.get("trackInfo");
                    
                    if (trackInfo != null) {
                        return String.format("%s - %s - %s", 
                            trackInfo.get("status"), 
                            trackInfo.get("dateTime"), 
                            trackInfo.get("office"));
                    }
                }
            }
            
            // 如果从佐川获取失败，尝试从斑马API获取
            try {
                String url = String.format(TRACKING_URL, trackingNumber);
                ResponseEntity<Map> response = restTemplate.getForEntity(url, Map.class);
                Map<String, Object> responseBody = response.getBody();
                if (responseBody != null && Integer.valueOf(0).equals(responseBody.get("code"))) {
                    return Optional.ofNullable(responseBody.get("data"))
                        .map(data -> (List<Map<String, Object>>) data)
                        .filter(list -> !list.isEmpty())
                        .map(list -> list.get(0))
                        .map(track -> (String) track.get("track"))
                        .orElse(null);
                }
            } catch (Exception e) {
                logger.error("从斑马API获取物流信息失败: {}", e.getMessage());
                // 继续处理，不中断流程
            }
        } catch (Exception e) {
            logger.error("获取物流信息失败: {}", e.getMessage());
            // 继续处理，不中断流程
        }
        
        // 如果所有尝试都失败，返回默认值
        return "暂无物流信息";
    }

    /**
     * 获取物流轨迹信息
     */
    @ApiOperation("获取物流轨迹信息")
    @GetMapping("/tracking/{trackingNumber}")
    public R<String> getTracking(@PathVariable("trackingNumber") String trackingNumber) {
        try {
            String trackInfo = getTrackingInfo(trackingNumber);
            return trackInfo != null ? R.ok(trackInfo) : R.fail("未找到物流信息");
        } catch (Exception e) {
            return R.fail("获取物流信息失败: " + e.getMessage());
        }
    }

    /**
     * 获取所有页的斑马订单
     */
    @ApiOperation("获取所有页的斑马订单")
    @GetMapping("/orders/all")
    @SuppressWarnings("unchecked")
    public DeferredResult<R<Map<String, Object>>> getAllOrders(
            @ApiParam("开始日期(yyyy-MM-dd)") @RequestParam(required = false) String startDate,
            @ApiParam("结束日期(yyyy-MM-dd)") @RequestParam(required = false) String endDate) {
        
        DeferredResult<R<Map<String, Object>>> deferredResult = new DeferredResult<>(9999000L);

        Thread thread = new Thread(() -> {
            try {
                if (deferredResult.isSetOrExpired()) return;

                // 获取第一页数据
                HttpEntity<String> entity = createHttpEntity();
                String url = buildApiUrl(1, DEFAULT_PAGE_SIZE, startDate, endDate);
                ResponseEntity<Map> response = restTemplate.exchange(url, HttpMethod.GET, entity, Map.class);
                Map<String, Object> responseBody = response.getBody();
                
                if (responseBody == null || !responseBody.containsKey("data")) {
                    setDeferredResult(deferredResult, R.fail("API返回数据无效"));
                    return;
                }

                Map<String, Object> dataMap = (Map<String, Object>) responseBody.get("data");
                int totalCount = ((Number) dataMap.getOrDefault("total", 0)).intValue();
                
                List<Map<String, Object>> orders = Optional.ofNullable(dataMap.get("list"))
                    .map(list -> (List<Map<String, Object>>) list)
                    .orElse(Collections.emptyList());
                
                // 处理订单数据
                List<Map<String, Object>> processedOrders = orders.stream()
                    .map(this::processOrderData)
                    .filter(order -> order != null)
                    .collect(Collectors.toList());

                // 计算分页信息
                int totalPages = (int) Math.ceil((double) totalCount / DEFAULT_PAGE_SIZE);
                // 修改hasMore判断逻辑，根据当前页数和总页数判断
                boolean hasMore = totalCount > 0 && 1 < totalPages;
                
                // 构建结果
                Map<String, Object> resultMap = new HashMap<>();
                resultMap.put("orders", processedOrders);
                resultMap.put("total", totalCount);
                resultMap.put("totalPages", totalPages);
                resultMap.put("hasMore", hasMore);
                resultMap.put("nextPage", 2);

                setDeferredResult(deferredResult, R.ok(resultMap));
            } catch (Exception e) {
                logger.error("获取订单数据失败: {}", e.getMessage());
                setDeferredResult(deferredResult, R.fail("获取订单失败: " + e.getMessage()));
            }
        });

        thread.setDaemon(true);
        thread.start();

        return deferredResult;
    }
    
    /**
     * 安全设置延迟结果
     */
    private void setDeferredResult(DeferredResult<R<Map<String, Object>>> deferredResult, R<Map<String, Object>> result) {
        if (!deferredResult.isSetOrExpired()) {
            deferredResult.setResult(result);
        }
    }

    /**
     * 获取下一页斑马订单
     */
    @ApiOperation("获取下一页斑马订单")
    @GetMapping("/orders/next")
    public DeferredResult<R<Map<String, Object>>> getNextPageOrders(
            @RequestParam(value = "page", defaultValue = "1") Integer page,
            @ApiParam("开始日期(yyyy-MM-dd)") @RequestParam(required = false) String startDate,
            @ApiParam("结束日期(yyyy-MM-dd)") @RequestParam(required = false) String endDate) {
        
        DeferredResult<R<Map<String, Object>>> deferredResult = new DeferredResult<>(999999999L);

        Thread thread = new Thread(() -> {
            try {
                if (deferredResult.isSetOrExpired()) return;
                
                // 获取总页数信息
                HttpEntity<String> entity = createHttpEntity();
                String url = buildApiUrl(1, DEFAULT_PAGE_SIZE, startDate, endDate);
                ResponseEntity<Map> countResponse = restTemplate.exchange(url, HttpMethod.GET, entity, Map.class);
                Map<String, Object> countResponseBody = countResponse.getBody();
                int totalPages = 1;
                
                if (countResponseBody != null && countResponseBody.containsKey("data")) {
                    Map<String, Object> dataMap = (Map<String, Object>) countResponseBody.get("data");
                    int totalCount = ((Number) dataMap.getOrDefault("total", 0)).intValue();
                    totalPages = (int) Math.ceil((double) totalCount / DEFAULT_PAGE_SIZE);
                }
                
                // 获取当前页数据
                R<List<Map<String, Object>>> pageResult = fetchOrdersFromApi(page, DEFAULT_PAGE_SIZE, startDate, endDate);
                
                if (pageResult.getCode() != 200) {
                    setDeferredResult(deferredResult, R.fail(pageResult.getMsg()));
                    return;
                }

                List<Map<String, Object>> pageData = pageResult.getData();
                
                // 修改hasMore判断逻辑，根据当前页数和总页数判断
                boolean hasMore = page < totalPages;
                
                Map<String, Object> resultMap = new HashMap<>();
                resultMap.put("orders", pageData);
                resultMap.put("hasMore", hasMore);
                resultMap.put("nextPage", page + 1);
                resultMap.put("totalPages", totalPages);
                
                setDeferredResult(deferredResult, R.ok(resultMap));
            } catch (Exception e) {
                logger.error("获取下一页订单失败: {}", e.getMessage());
                setDeferredResult(deferredResult, R.fail("获取订单失败: " + e.getMessage()));
            }
        });
        
        thread.setDaemon(true);
        thread.start();
        return deferredResult;
    }

    /**
     * 图片代理接口
     */
    @ApiOperation("图片代理接口")
    @GetMapping("/image-proxy")
    public void imageProxy(@RequestParam("url") String imageUrl, javax.servlet.http.HttpServletResponse response) {
        if (StringUtils.isEmpty(imageUrl)) {
            return;
        }
        try {
            HttpComponentsClientHttpRequestFactory factory = new HttpComponentsClientHttpRequestFactory();
            factory.setConnectTimeout(999999999);
            factory.setReadTimeout(999999999);
            RestTemplate proxyTemplate = new RestTemplate(factory);

            ResponseEntity<byte[]> imageResponse = proxyTemplate.getForEntity(imageUrl, byte[].class);
            byte[] imageBytes = imageResponse.getBody();
            if (imageBytes != null) {
                String contentType = Optional.ofNullable(imageResponse.getHeaders().getContentType())
                    .map(Object::toString)
                    .orElse("image/jpeg");
                    
                response.setContentType(contentType);
                response.setContentLength(imageBytes.length);
                response.getOutputStream().write(imageBytes);
                response.getOutputStream().flush();
            }
        } catch (Exception e) {
            logger.error("图片代理请求失败: {}", e.getMessage());
        }
    }
    
    /**
     * 手动刷新token接口
     */

    @GetMapping("/refresh-token")
    public R<String> manualRefreshToken() {
        refreshToken();
        return R.ok("Token刷新请求已执行");
    }
}   