package com.ruoyi.web.controller.tool;
import java.io.*;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Random;
import java.util.stream.Collectors;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.TimeUnit;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.context.request.async.DeferredResult;
import org.springframework.http.ResponseEntity;
import org.springframework.util.ResourceUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import com.ruoyi.common.annotation.Anonymous;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.R;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import us.codecraft.webmagic.Page;
import us.codecraft.webmagic.Site;
import us.codecraft.webmagic.Spider;
import us.codecraft.webmagic.processor.PageProcessor;
import us.codecraft.webmagic.selector.Html;
import us.codecraft.webmagic.downloader.HttpClientDownloader;
import us.codecraft.webmagic.proxy.Proxy;
import us.codecraft.webmagic.proxy.SimpleProxyProvider;
import us.codecraft.webmagic.scheduler.QueueScheduler;

import org.yaml.snakeyaml.Yaml;

import java.util.Objects;
import java.util.concurrent.TimeoutException;

/**
 * 亚马逊爬虫控制器 - 爬取价格和卖家信息
 * 
 * @author ruoyi
 */
@Api("亚马逊爬虫功能")
@RestController
@RequestMapping("/tool/webmagic")
@Anonymous
public class WebMagicController extends BaseController implements PageProcessor {

    private static final Logger logger = LoggerFactory.getLogger(WebMagicController.class);

    // 随机数生成器
    private final Random random = new Random();

    private static volatile Spider activeSpider = null;
    private static final Object spiderLock = new Object();

    // 站点配置
    private final Site site = Site.me()
            .setRetryTimes(3)
            .setSleepTime(1000 + random.nextInt(2000)) 
            .setTimeOut(99999999)
            .setUserAgent("Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/120.0.0.0 Safari/537.36")
            .addCookie("session-id", "358-1261309-0483141")
            .addCookie("session-id-time", "2082787201l")
            .addCookie("i18n-prefs", "JPY")
            .addCookie("lc-acbjp", "zh_CN")
            .addCookie("ubid-acbjp", "357-8224002-9668932");
    // 默认代理配置
    private static final String DEFAULT_PROXY_HOST = "127.0.0.1";
    private static final String DEFAULT_PROXY_PORT = "7890";

    private String currentProxyName = "";
    private String currentProxyHost = DEFAULT_PROXY_HOST;
    private int currentProxyPort = Integer.parseInt(DEFAULT_PROXY_PORT);

    // 存储所有代理节点信息
    private List<Map<String, Object>> proxyNodes = new ArrayList<>();
    
    // 爬取结果
    private Map<String, Object> resultMap = new HashMap<>();
    
    // Clash状态标志
    private static boolean clashRunning = false;

    /**
     * 初始化加载代理配置
     */
    public WebMagicController() {
        loadProxyConfig();
    }

    /**
     * 加载代理配置文件
     */
    @SuppressWarnings("unchecked")
    private void loadProxyConfig() {
        try {
        //   File configFile = ResourceUtils.getFile("classpath:test_proxy.yml");
         File configFile = new File("/www/java_mall/erp/config/test_proxy.yml");
            if (configFile.exists()) {
                Yaml yaml = new Yaml();
                try (InputStream inputStream = new FileInputStream(configFile)) {
                    Map<String, Object> config = yaml.load(inputStream);
                    if (config != null && config.containsKey("proxies")) {
                        List<Map<String, Object>> proxies = (List<Map<String, Object>>) config.get("proxies");
                        proxyNodes = proxies.stream()
                                .filter(proxy -> proxy != null && proxy.containsKey("name") && proxy.containsKey("server") && proxy.containsKey("port"))
                                .collect(Collectors.toList());
                    }
                }
            }
        } catch (Exception e) {
            logger.error("加载代理配置失败", e);
        }
    }

    /**
     * 获取所有可用代理节点
     */
    @ApiOperation("获取所有可用代理节点")
    @GetMapping("/proxies")
    public R<List<Map<String, String>>> getProxies() {
        List<Map<String, String>> result = proxyNodes.stream()
            .map(node -> {
                Map<String, String> proxyInfo = new HashMap<>();
                proxyInfo.put("name", (String) node.get("name"));
                proxyInfo.put("server", (String) node.get("server"));
                proxyInfo.put("port", String.valueOf(node.get("port")));
                proxyInfo.put("type", (String) node.get("type"));
                return proxyInfo;
            })
            .collect(Collectors.toList());
        return R.ok(result);
    }

    /**
     * 设置当前使用的代理节点
     */
    @ApiOperation("设置当前使用的代理节点")
    @PostMapping("/proxy/set")
    public R<Map<String, String>> setCurrentProxy(@RequestParam String proxyName) {
        // 查找匹配的代理节点
        Map<String, Object> targetProxy = proxyNodes.stream()
                .filter(node -> proxyName.equals(node.get("name")))
                .findFirst()
                .orElse(null);
        
        Map<String, String> result = new HashMap<>();
        
        if (targetProxy != null) {
            currentProxyName = (String) targetProxy.get("name");
            currentProxyHost = (String) targetProxy.get("server");
            currentProxyPort = ((Number) targetProxy.get("port")).intValue();

            result.put("name", currentProxyName);
            result.put("host", currentProxyHost);
            result.put("port", String.valueOf(currentProxyPort));

            return R.ok(result);
        } else {
            return R.fail("未找到指定的代理节点: " + proxyName);
        }
    }

    @Override
    public void process(Page page) {
        try {
            Html html = page.getHtml();
                String priceSymbol = html.xpath("//span[@class='a-price-symbol']/text()").toString();
                String priceWhole = html.xpath("//span[@class='a-price-whole']/text()").toString();

                if (priceSymbol != null && !priceSymbol.isEmpty() && priceWhole != null && !priceWhole.isEmpty()) {
                    resultMap.put("price", priceSymbol + priceWhole);
                }
            resultMap.put("seller", html.xpath("//a[@id='sellerProfileTriggerId']/text()").toString());
            resultMap.put("asin", html.xpath("//input[@id='ASIN']/@value").toString());
            String price = (String) resultMap.get("price");
            String seller = (String) resultMap.get("seller");
            Object retriesObj = page.getRequest().getExtra("retries");
            int retries = retriesObj == null ? 0 : (int) retriesObj;
            if ((price == null || price.isEmpty() || seller == null || seller.isEmpty()) && retries < 1) {
                String url = page.getUrl().toString();
                us.codecraft.webmagic.Request request = new us.codecraft.webmagic.Request(url);
                request.putExtra("retries", retries + 1);
                page.addTargetRequest(request);
                try {
                    Thread.sleep(1000 + random.nextInt(20000));
                } catch (InterruptedException e) {
                    Thread.currentThread().interrupt();
                }
            }
        } catch (Exception e) {
            logger.error("解析页面失败", e);
        }
    }
    @Override
    public Site getSite() {
        return site;
    }

    /**
     * 获取配置了代理的下载器
     */
    private HttpClientDownloader getProxyDownloader() {
        HttpClientDownloader httpClientDownloader = new HttpClientDownloader();
        clearSystemProxy();
            System.setProperty("http.proxyHost", DEFAULT_PROXY_HOST);
            System.setProperty("http.proxyPort", DEFAULT_PROXY_PORT);
            System.setProperty("https.proxyHost", DEFAULT_PROXY_HOST);
            System.setProperty("https.proxyPort", DEFAULT_PROXY_PORT);
            System.setProperty("http.nonProxyHosts", "localhost|127.0.0.1");
            httpClientDownloader.setProxyProvider(SimpleProxyProvider.from(
                new Proxy(DEFAULT_PROXY_HOST, Integer.parseInt(DEFAULT_PROXY_PORT))
            ));

        return httpClientDownloader;
    }

    private void clearSystemProxy() {
        System.clearProperty("http.proxyHost");
        System.clearProperty("http.proxyPort");
        System.clearProperty("https.proxyHost");
        System.clearProperty("socksProxyHost");
        System.clearProperty("socksProxyPort");
    }

    /**
     * 构建亚马逊产品URL
     */
    private String buildAmazonUrl(String asin) {
            asin = asin.replaceAll("[^a-zA-Z0-9]", "");
        return "https://www.amazon.co.jp/dp/" + asin;
    }

    /**
     * 启动 Clash 代理服务
     */
    public static boolean startClash() {
        logger.info("正在启动 Clash 代理服务...");
        try {
            if (isClashRunning()) {
                logger.info("Clash 已经在运行中，无需重复启动");
                clashRunning = true;
                return true;
            }
            
            ProcessBuilder pb = new ProcessBuilder("systemctl", "start", "clash");
            pb.redirectErrorStream(true);
            Process process = pb.start();
            
            int exitCode = process.waitFor();
     
                logger.info("Clash 启动成功");
                clashRunning = true;
                return true;
            
        } catch (Exception e) {
            logger.error("启动 Clash 时发生异常", e);
            return false;
        }
    }
    
    /**
     * 停止 Clash 代理服务
     */
    public static boolean stopClash() {
        logger.info("正在停止 Clash 代理服务...");
        try {
            if (!isClashRunning()) {
                logger.info("Clash 未在运行，无需停止");
                clashRunning = false;
                return true;
            }

            ProcessBuilder pb = new ProcessBuilder("systemctl", "stop", "clash");
            pb.redirectErrorStream(true);
            Process process = pb.start();
            
        process.waitFor();
         
                logger.info("Clash 停止成功");
                clashRunning = false;
                return true;
          
        } catch (Exception e) {
            logger.error("停止 Clash 时发生异常", e);
            return false;
        }
    }
    
    /**
     * 检查 Clash 是否在运行
     */
    public static boolean isClashRunning() {
        try {
            ProcessBuilder pb = new ProcessBuilder("bash", "-c", "pgrep -x clash");
            pb.redirectErrorStream(true);
            Process process = pb.start();
            
            BufferedReader reader = new BufferedReader(new InputStreamReader(process.getInputStream()));
            String line = reader.readLine();
            
            int exitCode = process.waitFor();
            return exitCode == 0 && line != null && !line.isEmpty();
        } catch (Exception e) {
            logger.error("检查 Clash 运行状态时发生异常", e);
            return false;
        }
    }
    
    /**
     * 获取 Clash 运行状态
     */
    @ApiOperation("获取Clash运行状态")
    @GetMapping("/clash/status")
    public R<Map<String, Object>> getClashStatus() {
        Map<String, Object> result = new HashMap<>();
        boolean running = isClashRunning();
        result.put("running", running);
        result.put("status", running ? "运行中" : "已停止");
        return R.ok(result);
    }
    
    /**
     * 手动控制 Clash 启动
     */
    @ApiOperation("启动Clash")
    @PostMapping("/clash/start")
    public R<Map<String, Object>> startClashManually() {
        Map<String, Object> result = new HashMap<>();
        boolean success = startClash();
        result.put("success", success);
        result.put("message", success ? "Clash 启动成功" : "Clash 启动失败");
        try {
            // 添加延迟，确保Clash完全启动
            Thread.sleep(3000);
        } catch (InterruptedException e) {
            Thread.currentThread().interrupt();
            logger.warn("Clash启动等待被中断", e);
        }
        return R.ok(result);
    }
    
    /**
     * 手动控制 Clash 停止
     */
    @ApiOperation("停止Clash")
    @PostMapping("/clash/stop")
    public R<Map<String, Object>> stopClashManually() {
        Map<String, Object> result = new HashMap<>();
        boolean success = stopClash();
        result.put("success", success);
        result.put("message", success ? "Clash 停止成功" : "Clash 停止失败");
        return R.ok(result);
    }

    /**
     * 批量爬取亚马逊产品信息
     */
    @ApiOperation("批量爬取亚马逊产品信息")
    @PostMapping("/batch")
    public DeferredResult<ResponseEntity<R<List<Map<String, Object>>>>> batchGetAmazonProductInfo(@RequestBody List<String> asinList) {
        DeferredResult<ResponseEntity<R<List<Map<String, Object>>>>> deferredResult = new DeferredResult<>(500000L);
        
        CompletableFuture.runAsync(() -> {
            List<Map<String, Object>> resultList = new ArrayList<>();
            List<String> failedAsins = new ArrayList<>();
            Map<String, Integer> asinIndexMap = new HashMap<>();
            
            try {
                int count = 0;
                for (String asin : asinList) {
                    if (asin == null || asin.trim().isEmpty()) {
                        continue;
                    }
                    String url = buildAmazonUrl(asin.trim());
                    resultMap = new HashMap<>();
                    resultMap.put("asin", asin);
                    Spider spider = null;
                    boolean success = false;
                    
                    try {
                        spider = Spider.create(this)
                                .addUrl(url)
                                .setDownloader(getProxyDownloader())
                                .thread(1);
                        
                  
                        synchronized (spiderLock) {
                            activeSpider = spider;
                        }
                        
                        spider.run();
                        
                        String price = (String) resultMap.get("price");
                        String resultAsin = (String) resultMap.get("asin");
                        
                        if (resultAsin == null || resultAsin.isEmpty() || price == null || price.isEmpty()) {
                            logger.info("ASIN: {} 首次爬取结果不完整，等待后重试", asin);
                            Thread.sleep(1000 + random.nextInt(20000));
                            
                            resultMap = new HashMap<>();
                            resultMap.put("asin", asin);
                            
                            spider = Spider.create(this)
                                    .addUrl(url)
                                    .setDownloader(getProxyDownloader())
                                    .thread(1);
                            
                            synchronized (spiderLock) {
                                activeSpider = spider;
                            }
                            
                            spider.run();
                            
                            price = (String) resultMap.get("price");
                            resultAsin = (String) resultMap.get("asin");
                            
                            if (resultAsin != null && !resultAsin.isEmpty() && price != null && !price.isEmpty()) {
                                success = true;
                                logger.info("ASIN: {} 重试成功", asin);
                            } else {
                                failedAsins.add(asin);
                                asinIndexMap.put(asin, resultList.size());
                            }
                        } else {
                            success = true;
                        }
                    } finally {
                        clearSystemProxy();
                        synchronized (spiderLock) {
                            if (activeSpider == spider) {
                                activeSpider = null;
                            }
                        }
                    }
                    
                    resultList.add(new HashMap<>(resultMap));
                    count++;
                }

                if (!failedAsins.isEmpty()) {
                    logger.info("发现{}个ASIN获取失败，将进行3次额外重试...", failedAsins.size());

                    for (int retryRound = 0; retryRound < 3; retryRound++) {
                        if (failedAsins.isEmpty()) {
                            break;
                        }
                        int waitSeconds = 30 + random.nextInt(21);
                        logger.info("第{}次重试，等待{}秒...", retryRound + 1, waitSeconds);
                        Thread.sleep(waitSeconds * 1000);
                        List<String> currentFailedAsins = new ArrayList<>(failedAsins);
                        
                        for (String failedAsin : currentFailedAsins) {
                            try {
                                String url = buildAmazonUrl(failedAsin);
                                
                                resultMap = new HashMap<>();
                                resultMap.put("asin", failedAsin);
                                
                                Spider spider = Spider.create(this)
                                        .addUrl(url)
                                        .setDownloader(getProxyDownloader())
                                        .thread(1);
                                
                                // 记录当前活动的爬虫
                                synchronized (spiderLock) {
                                    activeSpider = spider;
                                }
                                
                                spider.run();
                                
                                String price = (String) resultMap.get("price");
                                String resultAsin = (String) resultMap.get("asin");
                                
                                if (resultAsin != null && !resultAsin.isEmpty() && price != null && !price.isEmpty()) {
                                    logger.info("ASIN: {} 在第{}次重试中成功", failedAsin, retryRound + 1);
                                    
                                    Integer index = asinIndexMap.get(failedAsin);
                                    if (index != null) {
                                        resultList.set(index, new HashMap<>(resultMap));
                                    }
                                    failedAsins.remove(failedAsin);
                                } else {
                                    logger.warn("ASIN: {} 在第{}次重试中仍然失败", failedAsin, retryRound + 1);
                                }
                                Thread.sleep(15000 + random.nextInt(10000));
                                
                            } catch (Exception e) {
                                logger.error("重试ASIN: {} 时出错: {}", failedAsin, e.getMessage());
                            } finally {
                                clearSystemProxy();
                                synchronized (spiderLock) {
                                    activeSpider = null;
                                }
                            }
                        }
                        logger.info("第{}次重试完成，还有{}个ASIN未成功获取数据", retryRound + 1, failedAsins.size());
                    }

                    if (failedAsins.isEmpty()) {
                        logger.info("所有ASIN均成功获取数据");
                    } else {
                        logger.warn("经过3次重试，仍有{}个ASIN未能成功获取数据", failedAsins.size());
                    }
                }

                // 清除爬虫引用
                synchronized (spiderLock) {
                    activeSpider = null;
                }
                
                logger.info("批量爬取完成，结果数量: {}", resultList.size());
                deferredResult.setResult(ResponseEntity.ok(R.ok(resultList)));
            } catch (Exception e) {
                clearSystemProxy();
                logger.error("批量爬取失败", e);
                deferredResult.setResult(ResponseEntity.ok(R.fail("批量爬取失败")));
                synchronized (spiderLock) {
                    activeSpider = null;
                }
            }
        });
        return deferredResult;
    }
    
    /**
     * 测试代理节点延迟
     */
    @ApiOperation("测试代理节点延迟")
    @PostMapping("/proxy/test")
    public DeferredResult<ResponseEntity<R<List<Map<String, Object>>>>> testProxyDelay(@RequestBody List<String> proxyNames) {
        DeferredResult<ResponseEntity<R<List<Map<String, Object>>>>> deferredResult = new DeferredResult<>(600000L);
        
        // 测试前启动 Clash
        boolean clashStarted = startClash();
        if (!clashStarted) {
            logger.warn("启动 Clash 失败，将尝试继续测试...");
        }
        
        CompletableFuture.runAsync(() -> {
            List<Map<String, Object>> resultList = new ArrayList<>();
            
            try {
                for (int i = 0; i < proxyNames.size(); i += 2) {
                    final int currentIndex = i;
                    List<CompletableFuture<Map<String, Object>>> futures = new ArrayList<>();
                    
                    futures.add(CompletableFuture.supplyAsync(() -> testSingleProxy(proxyNames.get(currentIndex))));
                    
                    if (currentIndex + 1 < proxyNames.size()) {
                        futures.add(CompletableFuture.supplyAsync(() -> 
                            testSingleProxy(proxyNames.get(currentIndex + 1)))
                        );
                    }
                    
                    try {
                        CompletableFuture.allOf(futures.toArray(new CompletableFuture[0]))
                            .get(3, TimeUnit.SECONDS);
                    } catch (TimeoutException te) {
                        logger.debug("等待代理测试完成超时，这是正常现象");
                    }
                    for (CompletableFuture<Map<String, Object>> future : futures) {
                        try {
                            if (future.isDone() && !future.isCompletedExceptionally()) {
                                resultList.add(future.join());
                            } else if (!future.isDone()) {
                                Map<String, Object> timeoutResult = new HashMap<>();
                                timeoutResult.put("name", "unknown");
                                timeoutResult.put("status", "fail");
                                timeoutResult.put("message", "测试超时");
                                timeoutResult.put("delay", -1);
                                resultList.add(timeoutResult);
                            }
                        } catch (Exception ignored) {

                        }
                    }
                }
                
                deferredResult.setResult(ResponseEntity.ok(R.ok(resultList)));
            } catch (Exception e) {
                logger.error("测试代理延迟失败", e);
                deferredResult.setResult(ResponseEntity.ok(R.fail("测试代理延迟失败")));
            } finally {
                boolean clashStopped = stopClash();
                if (!clashStopped) {
                    logger.warn("停止 Clash 失败，请手动检查 Clash 状态");
                }
            }
        });
        
        return deferredResult;
    }
    
    /**
     * 测试单个代理节点延迟
     */
    private Map<String, Object> testSingleProxy(String proxyName) {
        Map<String, Object> result = new HashMap<>();
        result.put("name", proxyName);
        
        long startTime = System.currentTimeMillis();
        
        try {
            Map<String, Object> targetProxy = proxyNodes.stream()
                    .filter(node -> proxyName.equals(node.get("name")))
                    .findFirst()
                    .orElse(null);
            
            if (targetProxy == null) {
                result.put("status", "fail");
                result.put("message", "未找到指定的代理节点");
                result.put("delay", -1);
                return result;
            }

            String proxyHost = (String) targetProxy.get("server");
            int proxyPort = ((Number) targetProxy.get("port")).intValue();

            HttpClientDownloader testDownloader = new HttpClientDownloader();
            clearSystemProxy();
            testDownloader.setProxyProvider(SimpleProxyProvider.from(new Proxy(proxyHost, proxyPort)));

            Spider.create(new PageProcessor() {
                @Override
                public void process(Page page) {

                }
                
                @Override
                public Site getSite() {
                    return Site.me()
                            .setRetryTimes(0)
                            .setSleepTime(100)
                            .setTimeOut(3000); 
                }
            })
            .setDownloader(testDownloader)
            .addUrl("http://www.gstatic.com/generate_204")
            .thread(1)
            .run();
            
            long endTime = System.currentTimeMillis();
            
            result.put("status", "success");
            result.put("delay", endTime - startTime);
            
        } catch (Exception e) {
            result.put("status", "fail");
            result.put("message", "连接超时或失败");
            result.put("delay", -1);
        } finally {
            clearSystemProxy();
        }
        
        return result;
    }

    /**
     * 停止所有爬虫活动
     */
    @ApiOperation("停止所有爬虫活动")
    @PostMapping("/stop-crawling")
    public R<Map<String, Object>> stopAllCrawling() {
        Map<String, Object> result = new HashMap<>();
        boolean spiderStopped = false;
        
        // 停止活动的爬虫
        synchronized (spiderLock) {
            if (activeSpider != null) {
                try {
                    activeSpider.stop();
                    activeSpider = null;
                    spiderStopped = true;
                    logger.info("爬虫任务已终止");
                } catch (Exception e) {
                    logger.error("停止爬虫时出错", e);
                }
            }
        }
        boolean clashStopped = stopClash();
        
        result.put("success", spiderStopped || clashStopped);
        result.put("message", "爬虫和Clash已停止");
        
        return R.ok(result);
    }
} 